﻿using Infragistics.UltraChart.Core.Layers;
using Infragistics.UltraChart.Core.Primitives;
using Infragistics.UltraChart.Resources.Appearance;
using Infragistics.UltraChart.Shared.Styles;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace WindowsFormsApp1
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            DataTable dt = GenerateData();

            this.uniChart1.CompositeChart.ChartLayers.Clear();
            this.uniChart1.CompositeChart.Legends.Clear();
            this.uniChart1.CompositeChart.ChartAreas.Clear();
            this.uniChart1.CompositeChart.Series.Clear();

            this.uniChart1.TitleTop.Visible = false;
            this.uniChart1.TitleLeft.Visible = false;
            this.uniChart1.TitleRight.Visible = false;
            this.uniChart1.TitleBottom.Visible = false;

            this.uniChart1.ChartType = ChartType.Composite;

            ChartArea area = new ChartArea();

            this.uniChart1.CompositeChart.ChartAreas.Add(area);


            AxisItem xAxisColumn = new AxisItem(this.uniChart1, AxisNumber.X_Axis);
            AxisItem xAxisLine = new AxisItem(this.uniChart1, AxisNumber.X_Axis);
            AxisItem yAxis = new AxisItem(this.uniChart1, AxisNumber.Y_Axis);
            AxisItem yAxisLine = new AxisItem(this.uniChart1, AxisNumber.Y2_Axis);

            xAxisColumn.DataType = AxisDataType.String;
            xAxisColumn.SetLabelAxisType = SetLabelAxisType.GroupBySeries;
            xAxisColumn.Labels.ItemFormat = AxisItemLabelFormat.ItemLabel;

            // Added by IG
            xAxisColumn.Margin.Near.MarginType = LocationType.Pixels;
            xAxisColumn.Margin.Near.Value = 10;
            xAxisColumn.Margin.Far.MarginType = LocationType.Pixels;
            xAxisColumn.Margin.Far.Value =10;


            xAxisLine.DataType = AxisDataType.String;
            xAxisLine.SetLabelAxisType = SetLabelAxisType.ContinuousData;

            // Added by IG
            xAxisLine.Margin.Near.MarginType = LocationType.Pixels;
            xAxisLine.Margin.Near.Value = 10;
            xAxisLine.Margin.Far.MarginType = LocationType.Pixels;
            xAxisLine.Margin.Far.Value = 10;

            yAxis.DataType = AxisDataType.Numeric;
            yAxis.Labels.ItemFormat = AxisItemLabelFormat.DataValue;

            // Added by IG (for the sake of debugging)
            yAxis.RangeType = AxisRangeType.Custom;
            yAxis.RangeMin = 0;
            yAxis.RangeMax = 100;

            yAxisLine.DataType = AxisDataType.Numeric;
            yAxisLine.Labels.ItemFormat = AxisItemLabelFormat.DataValue;

            // Added by IG (for the sake of debugging)
            yAxisLine.RangeType = AxisRangeType.Custom;
            yAxisLine.RangeMin = 0;
            yAxisLine.RangeMax = 100;

            area.Axes.Add(xAxisColumn);
            area.Axes.Add(xAxisLine);
            area.Axes.Add(yAxis);
            area.Axes.Add(yAxisLine);

            NumericSeries seriesColumn = new NumericSeries();
            seriesColumn.Label = "plan";
            seriesColumn.PEs.Add(new PaintElement(Color.LightCoral));
            seriesColumn.Data.DataSource = dt;
            seriesColumn.Data.LabelColumn = "MM";
            seriesColumn.Data.ValueColumn = "plan_amt";

            NumericSeries seriesColumn2 = new NumericSeries();
            seriesColumn2.Label = "result";
            seriesColumn2.PEs.Add(new PaintElement(Color.Aquamarine));
            seriesColumn2.Data.DataSource = dt;
            seriesColumn2.Data.LabelColumn = "MM";
            seriesColumn2.Data.ValueColumn = "result_amt";

            NumericSeries seriesLine = new NumericSeries();
            seriesLine.Label = "achieve_rate";
            seriesLine.PEs.Add(new PaintElement(Color.Blue));
            seriesLine.Data.DataSource = dt;
            seriesLine.Data.LabelColumn = "MM";
            seriesLine.Data.ValueColumn = "achieve_rate";

            this.uniChart1.Series.AddRange(new Infragistics.UltraChart.Data.Series.ISeries[] { seriesLine, seriesColumn, seriesColumn2 });

            ChartLayerAppearance columnLayer = new ChartLayerAppearance();
            columnLayer.AxisX = xAxisColumn;
            columnLayer.AxisY = yAxis;
            columnLayer.ChartArea = area;
            columnLayer.ChartType = ChartType.ColumnChart;
            columnLayer.LegendItem = LegendItemType.Point;
            columnLayer.Series.Add(seriesColumn);
            columnLayer.Series.Add(seriesColumn2);
            columnLayer.SwapRowsAndColumns = true;

            ChartLayerAppearance lineLayer = new ChartLayerAppearance();
            lineLayer.AxisX = xAxisLine;
            lineLayer.AxisY = yAxisLine;
            lineLayer.ChartArea = area;
            lineLayer.ChartType = ChartType.LineChart;
            lineLayer.LegendItem = LegendItemType.Series;
            lineLayer.Series.Add(seriesLine);

            CompositeLegend legend = new CompositeLegend();
            legend.BoundsMeasureType = MeasureType.Percentage;
            legend.ChartLayers.Add(columnLayer);
            legend.ChartLayers.Add(lineLayer);

            this.uniChart1.CompositeChart.Legends.Add(legend);
            this.uniChart1.CompositeChart.ChartLayers.Add(columnLayer);
            this.uniChart1.CompositeChart.ChartLayers.Add(lineLayer);
            this.uniChart1.ColumnChart.SeriesSpacing = 1;
            this.uniChart1.Legend.Visible = true;
        }

        private DataTable GenerateData()
        {
            DataTable dt = new DataTable();

            /*
            seriesColumn.Data.DataSource = dt;
            seriesColumn.Data.LabelColumn = "MM";
            seriesColumn.Data.ValueColumn = "plan_amt";

            seriesColumn2.Data.DataSource = dt;
            seriesColumn2.Data.LabelColumn = "MM";
            seriesColumn2.Data.ValueColumn = "result_amt";

            seriesLine.Data.DataSource = dt;
            seriesLine.Data.LabelColumn = "MM";
            seriesLine.Data.ValueColumn = "achieve_rate";
            */

            // Columns
            dt.Columns.Add("MM", typeof(String));
            dt.Columns.Add("plan_amt", typeof(int));
            dt.Columns.Add("result_amt", typeof(int));
            dt.Columns.Add("achieve_rate", typeof(int));

            Random rnd = new Random();
            for(int i = 0; i < 5; i++)
            {
                dt.Rows.Add(new object[] { "MM" + i.ToString(), rnd.Next(80) + 10, rnd.Next(80) + 10, rnd.Next(80) + 10 });
            }

            return dt;
        }

        private void uniChart1_FillSceneGraph(object sender, Infragistics.UltraChart.Shared.Events.FillSceneGraphEventArgs e)
        {
            // Get a list of primitive boxes for the column chart
            List<Box> columnBoxes = e.SceneGraph.OfType<Box>().Where(b => b.Layer is ColumnLayer).OrderBy(b => b.rect.X).ToList();
            if (columnBoxes.Count == 0) return;

            // List of int for middle points of each group
            List<int> middlePointList = new List<int>();

            // Calculate middle points of each group
            // Assuming that each group has two columns
            for(int i = 0; i < columnBoxes.Count; i += 2)
            {
                // Calculate this middle point
                // Formula: (x1 + x2) / 2
                int x1 = columnBoxes[i].rect.X;
                int x2 = columnBoxes[i + 1].rect.X + columnBoxes[i + 1].rect.Width;
                middlePointList.Add((int)((double)(x1 + x2) / 2.0));
            }

            // Get a list of primitive Polyline for the line chart
            List<Polyline> line = e.SceneGraph.OfType<Polyline>().ToList();
            if (line.Count == 0) return;

            // Change the point of each data point of the line chart to its middle point
            for (int i = 0; i < line[0].points.Length; i++)
            {
                line[0].points[i].point.X = middlePointList[i];
            }
        }
    }
}
